/*******************************************************************************
    library for experimentation with linear RAAM

    by Michal Pokorny
    version 3.0
*******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>
#include <time.h>
#include "lraam.h"


#define DEFAULT_THETA 0.7
#define DEFAULT_ETA 0.01


// some parameters to be set by lraamlib_init
bool_t INITIALIZED = 0;
int MAX_DIMENSION = 16;
int ARITY = 2;
int MAX_STRUCT_LENGTH = 8192;
int MAX_TERM_LENGTH = 32;
int PRECISION = 3;
double EPS = 0.0001;
char FORMATSTR[47] = "%6.3lf\t";

// some parameters to be set randgen_init
bool_t RANDGENINITIALIZED = 0;
unsigned int RANDSEED = 0;
int RANDCALLS = 0;


/*******************
*                  *
*      init        *
*                  *
*******************/


// initialize random generator
void
randgen_init(unsigned int randseed, int randcalls) {

    int i;

    RANDSEED = randseed;
    srand(RANDSEED);
    
    RANDCALLS = randcalls;
    for (i=0; i<RANDCALLS; i++) rand();

    RANDGENINITIALIZED = 1;

    return;
}


// initialize working with library
void
lraamlib_init(int arity, int max_dimension, int max_struct_length, int max_term_length, int precision, int randseed) {
    int i;

    if (randseed < 0) randgen_init(time(NULL), 0);
    else randgen_init((unsigned int)randseed, 0);
    if (arity > 1) ARITY = arity;
    if (max_dimension > 0) MAX_DIMENSION = max_dimension;
    if (max_struct_length >= arity + 2) MAX_STRUCT_LENGTH = max_struct_length;
    if (max_term_length > 1) MAX_TERM_LENGTH = max_term_length;
    if (precision >= 2) {
	PRECISION = precision;
	EPS = 0.1;
	for (i=0; i<PRECISION; i++) EPS *= 0.1;
	sprintf(FORMATSTR, "%%%d.%dlf\t", PRECISION+3, PRECISION);
    }

    INITIALIZED = 1;

    return;    
}


/*******************
*                  *
*      misc        *
*                  *
*******************/


// return random value from <left, right> with d decimal numbers
double 
intrand(double left, double right, int d) {
    double pow = 1.0;
    double prec = 1.0;
    int i;
    
    assert(RANDGENINITIALIZED == 1);
    assert(d >= 0);
    
    RANDCALLS++;
    for (i=0; i<d; i++) { pow *= 10.0; prec *= 0.1; }

    // (-0.5, 0.5, 3):
    // <0;1) -> <0;1000) -> <0;1001) -> <-500;501) -> <-500;500> & Z -> <-0.500; 0.500>
    return ((int)(((right-left+prec)*pow*rand()/(RAND_MAX+1.0))+left*pow))*prec;
}


// permute given array
void 
permute(int * A, int n) {
    int i,j,tmp;
    
    assert(RANDGENINITIALIZED == 1);
    assert(A != NULL);
    assert(n >= 0);
    
    for (i=0; i<n; i++) {
	   j = (int)intrand(i, n-1, 0);
	   tmp = A[j]; A[j] = A[i]; A[i] = tmp;
    }

    return;
}


/*******************
*                  *
*     values       *
*                  *
*******************/


// print value of value_t variable to a file
void
value_fprintf(FILE * f, value_t value) {
    assert(f != NULL);
    
    fprintf(f, FORMATSTR, value);
    
    return;
}


// read value_t typed variable from file
error_t
value_fscanf(FILE * f, value_t * value) {
    error_t err = ERR_OK;
    
    assert(f != NULL);
    assert(value != NULL);

    if (fscanf(f, "%lf ", value) != 1) err = ERR_IO_FEW_PARAMS;
    
    return err;
}


/*******************
*                  *
*     vectors      *
*                  *
*******************/


// allocate memory for a vector
vector_t
vector_alloc(int dimension) {

    assert(dimension > 0);

    return (vector_t)calloc(dimension, sizeof(value_t));
}


// free memory allocated for a vector
void
vector_free(vector_t vector) {
    assert(vector != NULL);
    
    free(vector);
    
    return;
}


// reset vector to zero values
void
vector_reset(vector_t vector, int dimension) {

    assert(vector != NULL);
    assert(dimension > 0);
    
    memset(vector, 0, dimension * sizeof(value_t));

    return;
}


// set vector to a random valued one
void
vector_intrand(vector_t vector, int dimension, double left, double right) {
    int i;
    
    assert(RANDGENINITIALIZED == 1);
    assert(vector != NULL);
    assert(dimension > 0);

    for (i=0; i<dimension; i++) vector[i] = intrand(left, right, PRECISION);
    
    return;
}


// set all vector values to random value from <-1, 1>
void
vector_randomize(vector_t vector, int dimension) {

    assert(RANDGENINITIALIZED == 1);
    assert(vector != NULL);
    assert(dimension > 0);

    vector_intrand(vector, dimension, -1.0, 1.0);    

    return;
}


// make a copy of a vector
void 
vector_copy(vector_t dest, vector_t src, int dimension) {

    assert(dest != NULL);
    assert(src != NULL);
    assert(dimension > 0);

    memcpy(dest, src, dimension * sizeof(value_t));    
    
    return;
}


// allocate memory and make a copy of vector
vector_t
vector_clone(vector_t vector, int dimension) {
    vector_t clone;
    
    assert(vector != NULL);
    assert(dimension > 0);
    
    clone = vector_alloc(dimension);
    if (clone == NULL) return NULL;
    vector_copy(clone, vector, dimension);

    return clone;
}


// print vector to a file
void
vector_fprintf(FILE * f, vector_t vector, int dimension, char * eov) {
    int i;

    assert(f != NULL);
    assert(vector != NULL);
    assert(dimension > 0);
    assert(eov != NULL);
    
    for (i=0; i<dimension; i++) value_fprintf(f, vector[i]);
    fprintf(f, "%s", eov);
    
    return;
}


// write binary representation of vector to a file
void
vector_write(FILE * f, vector_t vector, int dimension) {

    assert(f != NULL);
    assert(vector != NULL);
    assert(dimension > 0);

    fwrite(vector, sizeof(value_t), dimension, f);
    
    return;
}


// read vector from a file
error_t
vector_fscanf(FILE * f, vector_t vector, int dimension) {
    error_t err = ERR_OK;
    int i;
    
    assert(f != NULL);
    assert(vector != NULL);
    assert(dimension > 0);
    
    for (i=0; i<dimension; i++) {
	err = value_fscanf(f, &vector[i]);
	if (err != ERR_OK) return err;
    }

    return err;
}


// read binary representation of vector from a file
error_t
vector_read(FILE * f, vector_t vector, int dimension) {

    assert(f != NULL);
    assert(vector != NULL);
    assert(dimension > 0);

    if (fread(vector, sizeof(value_t), dimension, f) != (unsigned int)dimension)
	return ERR_IO_FEW_PARAMS;
    
    return ERR_OK;
}


// return square of euclidian distance of two points in E_n
value_t
vector_sqredist(vector_t vect1, vector_t vect2, int dimension) {
    value_t edist;
    int i;
    
    assert(vect1 != NULL);
    assert(vect2 != NULL);
    assert(dimension > 0);
    
    edist = 0.0;
    for (i=0; i<dimension; i++) 
	edist += (vect1[i] - vect2[i])*(vect1[i] - vect2[i]);
    
    return edist;
}


/******************
*                 *
*     layers      *
*                 *
******************/


// set ith part of a layer
void 
layer_set_node(vector_t layer, vector_t repr, int i) {
    assert(INITIALIZED == 1);
    assert(layer != NULL);
    assert(repr != NULL);
    assert(i >= 0);
    assert(i < ARITY);
    
    vector_copy(layer+(i*MAX_DIMENSION), repr, MAX_DIMENSION);
    
    return;    
}


// get ith part of a layer
void 
layer_get_node(vector_t layer, vector_t repr, int i) {
    assert(INITIALIZED == 1);
    assert(layer != NULL);
    assert(repr != NULL);
    assert(i >= 0);
    assert(i < ARITY);
    
    vector_copy(repr, layer+(i*MAX_DIMENSION), MAX_DIMENSION);
    
    return;    
}


// print layer to a file
void
layer_fprintf(FILE * f, vector_t layer, int dimension, char * eov, char * eol) {
    int a;

    assert(INITIALIZED == 1);
    assert(f != NULL);
    assert(layer != NULL);
    assert(dimension > 0);
    assert(dimension <= MAX_DIMENSION);
    assert(eov != NULL);
    assert(eol != NULL);
    
    for (a=0; a<ARITY; a++) {
	vector_fprintf(f, layer+(a*MAX_DIMENSION), dimension, eov);
    }
    fprintf(f, "%s", eol);
    
    return;
}


// read layer from a file
error_t
layer_fscanf(FILE * f, vector_t layer, int dimension) {
    error_t err = ERR_OK;
    int a;
    
    assert(INITIALIZED == 1);
    assert(f != NULL);
    assert(layer != NULL);
    assert(dimension > 0);
    assert(dimension <= MAX_DIMENSION);
    
    for (a=0; a<ARITY; a++) {
	err = vector_fscanf(f, layer+(a*MAX_DIMENSION), dimension);
	if (err != ERR_OK) return err;
    }

    return err;
}


/******************
*                 *
*    matrices     *
*                 *
******************/


// allocate memory for a weight matrix
matrix_t
matrix_alloc() {
    matrix_t matrix;
    int a,i;
    
    assert(INITIALIZED == 1);

    if ((matrix = (matrix_t)calloc(ARITY, sizeof(value_t **))) == NULL) return NULL;
    for (a=0; a<ARITY; a++) {
        if ((matrix[a] = (value_t **)calloc(MAX_DIMENSION, sizeof(value_t *))) == NULL) return NULL;
        for (i=0; i<MAX_DIMENSION; i++) {
            if ((matrix[a][i] = (value_t *)calloc(MAX_DIMENSION, sizeof(value_t))) == NULL) return NULL;
        }
    }

    return matrix;
}


// free memory allocated for a weight matrix
void
matrix_free(matrix_t matrix) {
    int a, i;
    
    assert(INITIALIZED == 1);
    assert(matrix != NULL);
    
    for (a=0; a<ARITY; a++) {
	for (i=0; i<MAX_DIMENSION; i++) free(matrix[a][i]);
	free(matrix[a]);
    }
    free(matrix);

    return;
}


// reset all weights to zero value
void
matrix_reset(matrix_t matrix) {
    int a, i;
    
    assert(INITIALIZED == 1);
    assert(matrix != NULL);
    
    for (a=0; a<ARITY; a++)
    for (i=0; i<MAX_DIMENSION; i++)
	memset(matrix[a][i], 0, MAX_DIMENSION * sizeof(value_t));

    return;
}


// copy src matrix to dest matrix
void
matrix_copy(matrix_t dest, matrix_t src) {
    int a, i;
    
    assert(INITIALIZED == 1);
    assert(dest != NULL);
    assert(src != NULL);
    
    for (a=0; a<ARITY; a++)
    for (i=0; i<MAX_DIMENSION; i++)
	memcpy(dest[a][i], src[a][i], MAX_DIMENSION * sizeof(value_t));

    return;
}


// reset all weights to random value
void
matrix_intrand(matrix_t matrix, double left, double right) {
    int a, i, j;
    
    assert(RANDGENINITIALIZED == 1);
    assert(INITIALIZED == 1);
    assert(matrix != NULL);
    
    for (a=0; a<ARITY; a++)
    for (i=0; i<MAX_DIMENSION; i++)
    for (j=0; j<MAX_DIMENSION; j++)
	matrix[a][i][j] = intrand(left, right, PRECISION);

    return;
}


// reset all weights to random value from <-0.5; 0.5>
void
matrix_randomize(matrix_t matrix) {

    assert(RANDGENINITIALIZED == 1);
    assert(INITIALIZED == 1);
    
    matrix_intrand(matrix, -0.5, 0.5);

    return;
}


// prints weight matrix to a file
void
matrix_fprintf(FILE * f, matrix_t matrix, int dimension) {
    int a, i;

    assert(INITIALIZED == 1);
    assert(f != NULL);
    assert(matrix != NULL);
    assert(dimension > 0);
    assert(dimension <= MAX_DIMENSION);
    
    for (a=0; a<ARITY; a++) {
	for (i=0; i<dimension; i++) {
	    vector_fprintf(f, matrix[a][i], dimension, "");
	}
    }

    return;
}


// writes binary representation of  weight matrix to a file
void
matrix_write(FILE * f, matrix_t matrix, int dimension) {
    int a, i;

    assert(INITIALIZED == 1);
    assert(f != NULL);
    assert(matrix != NULL);
    assert(dimension > 0);
    assert(dimension <= MAX_DIMENSION);
    
    for (a=0; a<ARITY; a++) {
	for (i=0; i<dimension; i++) {
	    vector_write(f, matrix[a][i], dimension);
	}
    }

    return;
}


// read matrix from file
error_t
matrix_fscanf(FILE * f, matrix_t matrix, int dimension) {
    error_t err = ERR_OK;
    int a, i;
    
    assert(INITIALIZED == 1);
    assert(f != NULL);
    assert(matrix != NULL);
    assert(dimension > 0);
    assert(dimension <= MAX_DIMENSION);
    
    for (a=0; a<ARITY; a++) {
	for (i=0; i<dimension; i++) {
	    err = vector_fscanf(f, matrix[a][i], dimension);
	    if (err != ERR_OK) return err;
	}
    }

    return err;
}


// read binary representation of matrix from a file
error_t
matrix_read(FILE * f, matrix_t matrix, int dimension) {
    error_t err = ERR_OK;
    int a, i;
    
    assert(INITIALIZED == 1);
    assert(f != NULL);
    assert(matrix != NULL);
    assert(dimension > 0);
    assert(dimension <= MAX_DIMENSION);
    
    for (a=0; a<ARITY; a++) {
	for (i=0; i<dimension; i++) {
	    err = vector_read(f, matrix[a][i], dimension);
	    if (err != ERR_OK) return err;
	}
    }

    return err;
}


/*******************
*                  *
*     objects      *
*                  *
*******************/


// allocate memory for an object
object_t *
object_alloc(int type) {
    object_t * obj;    

    assert(INITIALIZED == 1);
    assert(type == TERMINAL || type == STRUCTURE);
    
    if ((obj = (object_t *)calloc(1, sizeof(object_t))) == NULL) return NULL;
    if ((obj->representation = vector_alloc(MAX_DIMENSION)) == NULL) { free(obj); return NULL; }
    if (type == TERMINAL) {
	if ((obj->terminal.label = (char *)calloc(MAX_TERM_LENGTH, sizeof(char))) == NULL) {
	    free(obj->representation); free(obj); return NULL;
	}
    } else {
	if ((obj->structure.description = (int *)calloc(MAX_STRUCT_LENGTH, sizeof(int))) == NULL) {
	    free(obj->representation); free(obj); return NULL;
	}
    }
    
    obj->type = type;
    object_reset(obj);

    return obj;
}


// allocate memory for and create copy of object
object_t *
object_clone(object_t * obj) {
    object_t * newobj;    

    assert(INITIALIZED == 1);
    
    if ((newobj = object_alloc(obj->type)) == NULL) return NULL;
    if (obj->type == TERMINAL) {
	strcpy(newobj->terminal.label, obj->terminal.label);
    } else {
	descr_copy(newobj->structure.description, obj->structure.description);
	newobj->structure.depth = obj->structure.depth;
    }
    
    vector_copy(newobj->representation, obj->representation, obj->dimension);
    newobj->dimension = obj->dimension;
    newobj->assigned = obj->assigned;

    return obj;
}


// release memory allocated for object
void
object_free(object_t * obj) {

    assert(INITIALIZED == 1);
    assert(obj != NULL);
    
    if (obj->type == TERMINAL) {
	free(obj->terminal.label);
    } else {
	free(obj->structure.description);
    }
    free(obj->representation);
    free(obj);

    return;
}


// reset objects properties but dont release memory
void
object_reset(object_t * obj) {

    assert(INITIALIZED == 1);
    assert(obj != NULL);
    
    if (obj->type == TERMINAL) {
	obj->terminal.label[0] = '\0';
    } else {
	obj->structure.description[0] = 0;
	obj->structure.depth = -1;
    }

    obj->dimension = MAX_DIMENSION;
    object_reset_representation(obj);    

    return;
}


// set description or label to an obejct
void
object_label(object_t * obj, void * label) {
    int length;

    assert(INITIALIZED == 1);
    assert(obj != NULL);
    assert(label != NULL);

    if (obj->type == TERMINAL) {
	length = strlen((char *)label);
	assert(length < MAX_TERM_LENGTH);	// leaving place for '\0'
	strncpy(obj->terminal.label, (char *)label, length+1);
    } else {
	length = descr_length((int *)label);
	assert(length < MAX_STRUCT_LENGTH);	// leaving place for LENGTH info
	descr_copy(obj->structure.description, (int *)label);
	obj->structure.depth = descr_depth(obj->structure.description);
    }

    return;
}


// assign representation to a terminal or a struct
void
object_assign_representation(object_t * obj, vector_t representation) {

    assert(obj != NULL);
    assert(representation != NULL);

    vector_copy(obj->representation, representation, obj->dimension);
    obj->assigned = 1;

    return;
}


// reset representation of a terminal or a struct
void
object_reset_representation(object_t * obj) {

    assert(INITIALIZED == 1);
    assert(obj != NULL);
    
    vector_reset(obj->representation, MAX_DIMENSION);
    obj->assigned = 0;

    return;
}


// change dimension of representation and fill new position with zeros
void
object_change_dimension(object_t * obj, int new_dimension) {
    int olddim;

    assert(INITIALIZED == 1);
    assert(obj != NULL);
    assert(new_dimension > 0);
    assert(new_dimension <= MAX_DIMENSION);
    
    olddim = obj->dimension;
    obj->dimension = new_dimension;
    if (new_dimension > olddim) {
	while (olddim < new_dimension) obj->representation[olddim++] = 0.0;
    } else {
	while (olddim > new_dimension) obj->representation[new_dimension++] = 0.0;
    }

    return;
}


// write terminal properties to a file
void
term_fprintf(FILE * f, object_t * term) {

    assert(f != NULL);
    assert(term != NULL);
    assert(term->type == TERMINAL);
    
    fprintf(f, "%s\t", term->terminal.label);
    vector_fprintf(f, term->representation, term->dimension, "\n");    

    return;
}


// create new terminal and read its properties from file
error_t
term_fscanf(FILE * f, int dimension, char * label, vector_t repr) {
    error_t err;

    assert(f != NULL);
    assert(dimension > 0);
    assert(dimension <= MAX_DIMENSION);
    assert(label != NULL);
    assert(repr != NULL);

    // reads label
    label[0] = '\0';
    if (fscanf(f, "%s", label) != 1) return ERR_IO_FEW_PARAMS;
    // reads representation
    if ((err = vector_fscanf(f, repr, dimension)) != ERR_OK) return err;

    return ERR_OK;
}


// print structure to a file
error_t
struct_fprintf(FILE * f, object_t * xstruct, set_t * xtermset, bool_t repr) {
    char * buffer;

    assert(f != NULL);
    assert(xstruct != NULL);
    assert(xstruct->type == STRUCTURE);
    assert(xtermset != NULL);
    assert(xtermset->type == TERMINAL);
    
    if ((buffer = (char *)calloc(MAX_STRUCT_LENGTH * MAX_TERM_LENGTH, sizeof(char))) == NULL) return ERR_NOT_ENOUGH_MEM;
    descr_to_str(xstruct->structure.description, buffer, xtermset);
    fprintf(f, "%s\t", buffer);
    free(buffer);
    if (repr) vector_fprintf(f, xstruct->representation, xstruct->dimension, "");
    fprintf(f, "\n");
    
    return ERR_OK;
}


/*******************
*                  *
*    description   *
*                  *
*******************/


// return length of a sctructure description
int
descr_length(int * descr) {

    assert(descr != NULL);
    assert(descr[0] < MAX_STRUCT_LENGTH);
    
    return descr[0];
}


// return depth of a structure
int
descr_depth(int * descr) {
    int depth = 0;
    int i;
    int d = 0;

    assert(descr != NULL);
    assert(descr[0] < MAX_STRUCT_LENGTH);
    
    if (descr[0] == 0) return -1;	// length == 0
    
    for (i=1; i<=descr[0]; i++) {
	if (descr[i] == LEFT_PAREN) {
	    d++;
	    if (d > depth) depth = d;
	} else if (descr[i] == RIGHT_PAREN) {
	    d--;
	    if (d < 0) return -1;
	}    
    }
    if (d != 0) return -1;
    
    return depth;
}


//append structure description src to end of dest
int 
descr_append(int * dest, int * src) {	// dest += src
    int ldest, lsrc;
    
    assert(dest != NULL);
    assert(src != NULL);
    assert(dest[0] + src[0] < MAX_STRUCT_LENGTH);
    
    ldest = dest[0];
    lsrc = src[0];
    
    memcpy(dest + ldest+1, src+1, lsrc * sizeof(int));
    dest[0] += lsrc;
    
    return dest[0];
}


// compares two structure descriptions
int
descr_cmp(int * descr1, int * descr2) {
    int i, min;
    
    assert(descr1 != NULL);
    assert(descr2 != NULL);
    assert(descr1[0] < MAX_STRUCT_LENGTH);
    assert(descr2[0] < MAX_STRUCT_LENGTH);

    min = descr1[0] < descr2[0] ? descr1[0] : descr2[0];
    
    for (i=1; i<=min; i++) {
	if (descr1[i] == descr2[i]) continue;
	return (descr1[i] - descr2[i]);			// not equal
    }

    if (descr1[0] == descr2[0]) return 0; 		// equal
    if (descr1[0] < descr2[0]) return -1;		// 1st shorter
    return 1;						// 2nd shorter
}


// copy structure description src to dest
int 
descr_copy(int * dest, int * src) {
    int length;
    
    assert(dest != NULL);
    assert(src != NULL);    
    assert(src[0] < MAX_STRUCT_LENGTH);

    length = src[0]+1;
    memcpy(dest, src, length * sizeof(int));

    return length;
}


// transform string to structure description
error_t
str_to_descr(char * str, int * descr, set_t * xtermset) {
    char * s;
    char * token;
    int id, strl, length;
    
    assert(str != NULL);
    assert(descr != NULL);
    assert(descr[0] < MAX_STRUCT_LENGTH);
    assert(xtermset != NULL);
    assert(xtermset->type == TERMINAL);
    
    // make a copy of str
    strl = strlen(str);
    if ((s = (char *)calloc(strl+1, sizeof(char))) == NULL) return ERR_NOT_ENOUGH_MEM;
    strncpy(s, str, strl+1);
    
    // tokenize str to transform into description
    token = strtok(s," ");
    length = 0;
    while (token != NULL) {
	length++;
	if (!strcmp(token, "(")) {
	    descr[length] = LEFT_PAREN;	    
	} else if (!strcmp(token, ")")) {
	    descr[length] = RIGHT_PAREN;
	} else {
	    id = set_get_term_by_label(xtermset, token);
	    if (id == -1) { free(s); return ERR_SYNTAX; }
	    descr[length] = id;
	}
	token = strtok(NULL," ");	
    }
    descr[0] = length; 

    return ERR_OK;
}


// transform structure description to a string
void
descr_to_str(int * descr, char * str, set_t * xtermset) {
    int i;
    object_t * obj;
    
    assert(descr != NULL);
    assert(descr[0] < MAX_STRUCT_LENGTH);
    assert(str != NULL);
    assert(xtermset != NULL);
    assert(xtermset->type == TERMINAL);

    str[0] = '\0';
    
    for (i=1; i<=descr[0]; i++) {
	if (descr[i] == LEFT_PAREN) {
	    strcat(str, "( ");
	} else if (descr[i] == RIGHT_PAREN) {
	    strcat(str, ") ");
	} else {
	    obj = set_get_object_by_id(xtermset, descr[i]);
	    strcat(str, obj->terminal.label);
	    strcat(str, " ");
	}
    }
    
    return;
}


// temporary function called by descr_validate()
static error_t
validate(int ** descr) {
    int i;
    error_t err;

    // single terminal
    if (**descr >= 0) return ERR_OK;
    
    // left parenthesis required
    if (**descr != LEFT_PAREN) return ERR_SYNTAX;
    (*descr)++;

    for (i=0; i<ARITY; i++) {
	err = validate(descr);
	if (err != ERR_OK) return err;
    }

    // right parenthesis required
    if (**descr != RIGHT_PAREN) return ERR_SYNTAX;
    (*descr)++;

    return ERR_OK;
}


// validate structure description with respect to known ARITY
error_t
descr_validate(int * descr) {
    int ** ip;

    assert(INITIALIZED == 1);
    assert(descr != NULL);
    assert(descr[0] < MAX_STRUCT_LENGTH);

    ip = &descr;
    if (**ip < ARITY+2) return ERR_SYNTAX;	// structure must be at least tree of height 1
    (*ip)++;
    
    return validate(ip);
}


/*******************
*                  *
*        set       *
*                  *
*******************/


// allocate memory for a set
set_t *
set_alloc(int init_array_length, int type) {
    int i, j;
    set_t * xset;
    
    assert(INITIALIZED == 1);
    assert(init_array_length > 0);
    assert(type == TERMINAL || type == STRUCTURE);

    if ((xset = (set_t *)calloc(1, sizeof(set_t))) == NULL) return NULL;
    if ((xset->objects = (object_t **)calloc(init_array_length, sizeof(object_t *))) == NULL) {
	free(xset); return NULL;
    }
    for (i=0; i<init_array_length; i++) {
	if ((xset->objects[i] = object_alloc(type)) == NULL) {
	    for (j=0; j<i; j++) object_free(xset->objects[j]);
	    free(xset->objects); free(xset); return NULL;
	}
    }

    xset->type = type;
    xset->array_length = init_array_length;
    xset->no_of_objects = 0;
    xset->dimension = MAX_DIMENSION;

    return xset;
}


// reallocate memory for an object array in a set
error_t
set_realloc(set_t * xset) {
    object_t ** tmp;
    int i, j, n;
    int type;
    
    assert(xset != NULL);

    n = xset->array_length;
    type = xset->type;
    if ((tmp = (object_t**)calloc(n*2, sizeof(object_t *))) == NULL) return ERR_NOT_ENOUGH_MEM;
    for (i=n; i<2*n; i++) {
	if ((tmp[i] = object_alloc(type)) == NULL) {
	    for (j=n; j<i; j++) object_free(tmp[j]);
	    free(tmp); return ERR_NOT_ENOUGH_MEM;
	}
    }

    for (i=0; i<n; i++) tmp[i] = xset->objects[i];
    free(xset->objects);
    xset->objects = tmp;
    xset->array_length *= 2;

    return ERR_OK;
}


// free memory allocated for set and objects in set
void
set_free(set_t * xset) {
    int i;
    
    assert(xset != NULL);
    
    for (i=0; i<xset->array_length; i++) object_free(xset->objects[i]);
    free(xset->objects);
    free(xset);
    
    return;    
}


// reset all objects in a set, no_of_objects := 0
void
set_reset(set_t * xset) {
    int i;    

    assert(xset != NULL);
    for (i=0; i<xset->array_length; i++) object_reset(xset->objects[i]);
    xset->no_of_objects = 0;
    xset->dimension = MAX_DIMENSION;    
    
    return;
}


// change dimension of set and of all objects in the set
void
set_change_dimension(set_t * xset, int new_dimension) {
    int i;

    assert(INITIALIZED == 1);
    assert(xset != NULL);
    assert(new_dimension > 0);
    assert(new_dimension <= MAX_DIMENSION);
    
    for (i=0; i<xset->array_length; i++) object_change_dimension(xset->objects[i], new_dimension);
    xset->dimension = new_dimension;

    return;
}


// "add" an object to a set
int
set_add_object(set_t * xset, void * label, vector_t repr) {    
    object_t * obj;

    assert(xset != NULL);
    assert(label != NULL);
    // repr may be NULL. If NULL, do not assign representation
        
    if (xset->array_length == xset->no_of_objects) 
	if (set_realloc(xset) != ERR_OK) return -1;

    obj = xset->objects[xset->no_of_objects];
    object_label(obj, label);
    if (repr != NULL) object_assign_representation(obj, repr);
    
    return (xset->no_of_objects++);
}


/*******************
*                  *
*    set misc      *
*                  *
*******************/


// return id of terminal specified by label within the set of terminals, or -1 of not found
int
set_get_term_by_label(set_t * xtermset, char * label) {
    int id;
    int i;
    
    assert(xtermset != NULL);
    assert(xtermset->type == TERMINAL);
    assert(label != NULL);
    
    id = -1;
    for (i=0; i<xtermset->no_of_objects; i++) {
	if (!strcmp(xtermset->objects[i]->terminal.label, label)) { id = i; break; }
    }

    return id;
}


// match vector with possible terminal(s)
int
set_get_term_by_repr(set_t * xtermset, vector_t repr, value_t theta) {
    int i, id, dim;
    value_t edist;
    
    assert(xtermset != NULL);
    assert(xtermset->type == TERMINAL);
    assert(repr != NULL);
    assert(theta >= 0);
    
    dim = xtermset->dimension;
    id = NO_MATCHING_TERMINAL;
    for (i=0; i<xtermset->no_of_objects; i++) {
	edist = sqrt(vector_sqredist(repr, xtermset->objects[i]->representation, dim));
	if (theta - edist > EPS) {
	    if (id != NO_MATCHING_TERMINAL) return TWO_MATCHING_TERMINALS;
	    else id = i;
	}
    }

    return id;
}


// return pointer to ith object in a set
object_t *
set_get_object_by_id(set_t * xset, int i) {
    
    assert(xset != NULL);

    if (i<0 || i>=xset->no_of_objects) return NULL;
    
    return xset->objects[i];
}


// save terminals to a file
void set_save_terminals(FILE * f, set_t * xtermset) {
    int i;

    assert(f != NULL);
    assert(xtermset != NULL);
    assert(xtermset->type == TERMINAL);
    
    fprintf(f, "%d\n", xtermset->no_of_objects);
    for (i=0; i<xtermset->no_of_objects; i++) {
	term_fprintf(f, xtermset->objects[i]);
    }
    
    return;
}


// save structures to a file
void set_save_structures(FILE * f, set_t * xdataset, set_t * xtermset, bool_t repr) {
    int i;

    assert(f != NULL);
    assert(xdataset != NULL);
    assert(xdataset->type == STRUCTURE);
    assert(xtermset != NULL);
    assert(xtermset->type == TERMINAL);
    
    fprintf(f, "%d\n", xdataset->no_of_objects);
    for (i=0; i<xdataset->no_of_objects; i++) {
	struct_fprintf(f, xdataset->objects[i], xtermset, repr);
    }
    
    return;
}


// load terminals and their representations from a file
set_t *
set_load_terminals(FILE * f, int dimension) {
    int i, n;
    error_t err = ERR_OK;
    set_t * xset;

    assert(f != NULL);
    assert(dimension > 0);
    assert(dimension <= MAX_DIMENSION);

    fscanf(f, "%d\n", &n);
    if ((xset = set_alloc(n, TERMINAL)) == NULL) return NULL;
    set_change_dimension(xset, dimension);
    for (i=0; i<n; i++) {
	err = term_fscanf(f, dimension, xset->objects[i]->terminal.label, xset->objects[i]->representation);
	if (err != ERR_OK) { set_free(xset); return NULL; }
	xset->objects[i]->assigned = 1;
    }
    xset->no_of_objects = n;
    
    return xset;
}


// loads structures from a file and collects all terminals
set_t *
set_load_structures(FILE * f, set_t * xtermset) {
    error_t err;
    int i, n;
    set_t * xset;
    object_t * obj;
    char * buffer;

    assert(f != NULL);
    assert(xtermset != NULL);
    assert(xtermset->type == TERMINAL);
    
    if ((buffer = (char *)calloc(MAX_STRUCT_LENGTH * MAX_TERM_LENGTH, sizeof(char))) == NULL) return NULL;
    
    fscanf(f, "%d\n", &n);
    if ((xset = set_alloc(n, STRUCTURE)) == NULL) return NULL;
    
    for (i=0; i<n; i++) {
	if (fscanf(f,"%[^\n\t]\n", buffer) != 1) { set_free(xset); return NULL; }
	obj = xset->objects[i];
	err = str_to_descr(buffer, obj->structure.description, xtermset);
	if (err == ERR_OK) {
	    obj->structure.depth = descr_depth(obj->structure.description);
	} else {
	    set_free(xset);
	    return NULL;
	}
    }
    xset->no_of_objects = n;

    return xset;
}


/*******************
*                  *
*   linear RAAM    *
*                  *
*******************/


// set all activities on input_layer to 0
void
lraam_reset_input_layer(lraam_t * xraam) {

    assert(INITIALIZED == 1);
    assert(xraam != NULL);

    vector_reset(xraam->input_layer, ARITY * MAX_DIMENSION);    
	
    return;	
}


// set all activities on output_layer to 0
void
lraam_reset_output_layer(lraam_t * xraam) {

    assert(INITIALIZED == 1);
    assert(xraam != NULL);
    
    vector_reset(xraam->output_layer, ARITY * MAX_DIMENSION);    
	
    return;	
}


// set all activities on compression_layer to 0
void
lraam_reset_compression_layer(lraam_t * xraam) {
    
    assert(INITIALIZED == 1);
    assert(xraam != NULL);
    
    vector_reset(xraam->compression_layer, MAX_DIMENSION);
	
    return;	
}


// set all weights to random value from <-0.5; 0.5>
void
lraam_randomize_weights(lraam_t * xraam) {
    
    assert(RANDGENINITIALIZED == 1);
    assert(xraam != NULL);
    
    matrix_randomize(xraam->W);
    
    return;
}


// linear RAAM allocation
lraam_t *
lraam_alloc() {
    lraam_t * xraam;

    assert(INITIALIZED == 1);

    // lraam ptr allocation
    if ((xraam = (lraam_t *)calloc(1, sizeof(lraam_t))) == NULL) return NULL;
    
    // params
    xraam->dimension = MAX_DIMENSION;
    xraam->theta = DEFAULT_THETA;
    xraam->eta = DEFAULT_ETA;
    
    // weight matrix
    if ((xraam->W = matrix_alloc()) == NULL) {
	free(xraam);
	return NULL;
    }
    
    // input layer
    if ((xraam->input_layer = vector_alloc(ARITY * MAX_DIMENSION)) == NULL) {
	matrix_free(xraam->W);
	free(xraam);
	return NULL;
    }

    // output layer
    if ((xraam->output_layer = vector_alloc(ARITY * MAX_DIMENSION)) == NULL) {
	matrix_free(xraam->W);
	vector_free(xraam->input_layer);
	free(xraam);
	return NULL;
    }

    // compression layer
    if ((xraam->compression_layer = vector_alloc(MAX_DIMENSION)) == NULL) {
	matrix_free(xraam->W);
	vector_free(xraam->input_layer);
	vector_free(xraam->output_layer);
	free(xraam);
	return NULL;
    }

    lraam_reset_input_layer(xraam);
    lraam_reset_output_layer(xraam);
    lraam_reset_compression_layer(xraam);
    
    return xraam;
}


// load and allocate linear RAAM from file -- use rather "experiment_load"
lraam_t *
lraam_load(FILE * f) {
    lraam_t * xraam;
    int arity, dimension;
    value_t theta, eta;

    assert(INITIALIZED == 1);    
    assert(f != NULL);
    
    if (fscanf(f, "%d\n", &arity) != 1) return NULL;
    if (arity != ARITY) return NULL;
    if (fscanf(f, "%d\n", &dimension) != 1) return NULL;
    if (dimension <= 0 || dimension > MAX_DIMENSION) return NULL;
    
    if ((xraam = lraam_alloc()) == NULL) return NULL;
    lraam_change_dimension(xraam, dimension);

    value_fscanf(f, &theta);
    value_fscanf(f, &eta);
    lraam_set_theta(xraam, theta);
    lraam_set_eta(xraam, eta);
    
    if (matrix_fscanf(f, xraam->W, dimension) != ERR_OK) {
	lraam_free(xraam);
	return NULL;
    }
    
    return xraam;
}


// save linear RAAM to file -- use rather "experiment_save"
void
lraam_save(FILE * f, lraam_t * xraam) {
    
    assert(INITIALIZED == 1);
    assert(f != NULL);
    assert(xraam != NULL);
    
    fprintf(f, "%d\n", ARITY);
    fprintf(f, "%d\n", xraam->dimension);
    value_fprintf(f, xraam->theta);
    fprintf(f, "\n");
    value_fprintf(f, xraam->eta);
    fprintf(f, "\n");
    matrix_fprintf(f, xraam->W, xraam->dimension);
    
    return;
}


// free memory allocated for a lraam
void 
lraam_free(lraam_t * xraam) {    

    assert(INITIALIZED == 1);
    assert(xraam != NULL);

    matrix_free(xraam->W);
    vector_free(xraam->input_layer);
    vector_free(xraam->output_layer);
    vector_free(xraam->compression_layer);
    free(xraam);

    return;
}


// change dimension and update model
void
lraam_change_dimension(lraam_t * xraam, int new_dimension) {
    int olddim;
    int a, i;

    assert(INITIALIZED == 1);
    assert(xraam != NULL);
    assert(new_dimension > 0);
    assert(new_dimension <= MAX_DIMENSION);
    
    // cleaned all but used cells for vectors    
    // example: olddim = 5
    // abcde????? -> abcde00000
    olddim = xraam->dimension;
    for (a=0; a<ARITY; a++) {
	for (i=olddim; i<MAX_DIMENSION; i++) {
	    xraam->input_layer[(a*MAX_DIMENSION)+i] = 0.0;
	    xraam->output_layer[(a*MAX_DIMENSION)+i] = 0.0;
	}
    }
    for (i=olddim; i<MAX_DIMENSION; i++) xraam->compression_layer[i] = 0.0;
    
    // in case old dimension is > than new dimension, it's necessary to
    // clean rest of cells    
    // example: newdim = 3
    // abcde00000 -> abc0000000
    for (a=0; a<ARITY; a++) {
	for (i=new_dimension; i<olddim; i++) {
	    xraam->input_layer[(a*MAX_DIMENSION)+i] = 0.0;
	    xraam->output_layer[(a*MAX_DIMENSION)+i] = 0.0;
	}
    }
    for (i=new_dimension; i<olddim; i++)
	xraam->compression_layer[i] = 0.0;

    xraam->dimension = new_dimension;

    return;
}


// sets theta parameter
void 
lraam_set_theta(lraam_t * xraam, value_t xtheta) {

    assert(xraam != NULL);
    
    xraam->theta = xtheta;
    
    return;
}


// returns current value of theta parameter
value_t
lraam_get_theta(lraam_t * xraam) {

    assert(xraam != NULL);
    
    return xraam->theta;
}


// sets eta parameter
void 
lraam_set_eta(lraam_t * xraam, value_t xeta) {

    assert(xraam != NULL);
    
    xraam->eta = xeta;
    
    return;
}


// returns current value of eta parameter
value_t
lraam_get_eta(lraam_t * xraam) {

    assert(xraam != NULL);
    
    return xraam->eta;
}


// copy representation to ith node
void lraam_set_node(lraam_t * xraam, vector_t repr, int i) {

    assert(xraam != NULL);

    layer_set_node(xraam->input_layer, repr, i);

    return;
}


// get representation from ith node of output layer
void lraam_get_modif_node(lraam_t * xraam, vector_t repr, int i) {

    assert(xraam != NULL);

    layer_get_node(xraam->output_layer, repr, i);

    return;
}


// place compressed representation on compression layer
void lraam_set_compressed(lraam_t * xraam, vector_t repr) {

    assert(xraam != NULL);

    vector_copy(xraam->compression_layer, repr, MAX_DIMENSION);

    return;
}


// get representation from compression layer
void lraam_get_compressed(lraam_t * xraam, vector_t repr) {

    assert(xraam != NULL);

    vector_copy(repr, xraam->compression_layer, MAX_DIMENSION);

    return;
}


// encode nodes' representations into representation of whole
void 
lraam_encode(lraam_t * xraam) {
    int a, i, j;
    int dim;

    assert(xraam != NULL);    

    dim = xraam->dimension;
    lraam_reset_compression_layer(xraam);
    for (a=0; a<ARITY; a++)
    for (i=0; i<dim; i++)
    for (j=0; j<dim; j++)
        xraam->compression_layer[i] += xraam->W[a][i][j] * xraam->input_layer[(a*MAX_DIMENSION)+j];

    return;
}


// reconstructs nodes' representations from encoded representation
void 
lraam_decode(lraam_t * xraam) {
    int a, i, j;
    int dim;

    assert(xraam != NULL);

    dim = xraam->dimension;
    lraam_reset_output_layer(xraam);
    for (a=0; a<ARITY; a++)
    for (i=0; i<dim; i++) 
    for (j=0; j<dim; j++)
	xraam->output_layer[(a*MAX_DIMENSION)+j] += xraam->W[a][i][j] * xraam->compression_layer[i];

    return;
}

// change weights by modified Oja's rule
void 
lraam_update_weights(lraam_t * xraam) {
    int a, i, j;
    int dim;
    double delta;
    
    assert(xraam != NULL);
    
    dim = xraam->dimension;
    for (a=0; a<ARITY; a++)
    for (i=0; i<dim; i++)
    for (j=0; j<dim; j++) {
	delta = xraam->eta * xraam->compression_layer[i] 
	    * (xraam->input_layer[(a*MAX_DIMENSION)+j] - xraam->output_layer[(a*MAX_DIMENSION)+j]);
	xraam->W[a][i][j] += delta;
    }

    return;
}


/*******************
*                  *
*      backup      *
*                  *
*******************/

/* next two functions provide what is known as Serialization in Java */


// loads conditions of experiment as it was saved
// makes an initialization (calls lraamlib_init())
error_t
experiment_load(FILE * f, lraam_t ** xraam, set_t ** xdataset, set_t ** xtermset) 
{
    char * s;
    int arity = 0, dimension = 0;
    int randseed = -1, randcalls = -1;
    int precision = -1;
    error_t err;
    char version[47];

    assert(f != NULL);
    assert(xraam != NULL);
    assert(xdataset != NULL);
    assert(xtermset != NULL);
    
    // memory allocation
    if ((s = (char *)calloc(65536, sizeof(char))) == NULL)
	return ERR_NOT_ENOUGH_MEM;

    // version - taken into account from v1.1
    if (fscanf(f, "%[^=]=", s) != 1) return ERR_SYNTAX;
    if (strcmp(s, "version")) {	// version 1.0
	strcpy(version, "1.0");
	// dont use s token here
    } else {
	if (fscanf(f, "%s\n", version) != 1) return ERR_SYNTAX; // version number
	if (fscanf(f, "%[^=]=", s) != 1) return ERR_SYNTAX; // next line 
    }
    
    // randseed
    // s token read already
    if (strcmp(s, "randseed")) return ERR_SYNTAX;
    if (fscanf(f, "%d\n", &randseed) != 1) return ERR_SYNTAX;
    if (randseed < 0) return ERR_SYNTAX;
    // randcalls
    if (fscanf(f, "%[^=]=", s) != 1) return ERR_SYNTAX;
    if (strcmp(s, "randcalls")) return ERR_SYNTAX;
    if (fscanf(f, "%d\n", &randcalls) != 1) return ERR_SYNTAX;
    if (randcalls < 0) return ERR_SYNTAX;
    // precision
    if (strcmp(version, "1.0")) { // from version v1.1
	if (fscanf(f, "%[^=]=", s) != 1) return ERR_SYNTAX;
	if (strcmp(s, "precision")) return ERR_SYNTAX;
	if (fscanf(f, "%d\n", &precision) != 1) return ERR_SYNTAX;
	if (precision < 0) precision = -1;
    }
    // arity
    if (fscanf(f, "%[^=]=", s) != 1) return ERR_SYNTAX;
    if (strcmp(s, "arity")) return ERR_SYNTAX;
    if (fscanf(f, "%d\n", &arity) != 1) return ERR_SYNTAX;
    if (arity < 2) return ERR_SYNTAX;
    // dimension
    if (fscanf(f, "%[^=]=", s) != 1) return ERR_SYNTAX;
    if (strcmp(s, "dimension")) return ERR_SYNTAX;
    if (fscanf(f, "%d\n", &dimension) != 1) return ERR_SYNTAX;
    if (dimension <= 0) return ERR_SYNTAX;
    
    lraamlib_init(arity, dimension, -1, -1, precision, -1);
    randgen_init(randseed, randcalls);
    
    // raam allocation
    *xraam = lraam_alloc();
    if (*xraam == NULL) return ERR_NOT_ENOUGH_MEM;
    
    // weight matrix
    if (fscanf(f, "%[^=]=", s) != 1) return ERR_SYNTAX;
    if (strcmp(s, "weights")) return ERR_SYNTAX;
    err = matrix_read(f, (*xraam)->W, dimension);
    if (err != ERR_OK) return ERR_SYNTAX;
    fscanf(f, "\n");
    
    // termset
    if (fscanf(f, "%[^=]=", s) != 1) return ERR_SYNTAX;
    if (strcmp(s, "termset")) return ERR_SYNTAX;
    (*xtermset) = set_load_terminals(f, dimension);
    if (*xtermset == NULL) return ERR_SYNTAX;
    
    // dataset
    if (fscanf(f, "%[^=]=", s) != 1) return ERR_SYNTAX;
    if (strcmp(s, "dataset")) return ERR_SYNTAX;
    (*xdataset) = set_load_structures(f, *xtermset);
    if (*xdataset == NULL) return ERR_SYNTAX;
    
    // free temporary memory
    free(s);

    return ERR_OK;
}


// saves conditions of experiment
void
experiment_save(FILE * f, lraam_t * xraam, set_t * xdataset, set_t * xtermset, bool_t readable) 
{

    assert(f != NULL);
    assert(xraam != NULL);
    assert(xdataset != NULL);
    assert(xdataset->type == STRUCTURE);
    assert(xtermset != NULL);
    assert(xtermset->type == TERMINAL);
    assert(xraam->dimension == xdataset->dimension);
    assert(xraam->dimension == xtermset->dimension);
    
    fprintf(f, "version=1.1\n");
    fprintf(f, "randseed=%d\n", RANDSEED);
    fprintf(f, "randcalls=%d\n", RANDCALLS);
    fprintf(f, "precision=%d\n", PRECISION);
    fprintf(f, "arity=%d\n", ARITY);
    fprintf(f, "dimension=%d\n", xraam->dimension);

    fprintf(f, "weights=");
    if (readable) {
	fprintf(f, "\n");
	matrix_fprintf(f, xraam->W, xraam->dimension);
    } else {
	matrix_write(f, xraam->W, xraam->dimension);
    }
    fprintf(f, "\n");

    fprintf(f, "termset=");
    set_save_terminals(f, xtermset);
    
    fprintf(f, "dataset=");
    set_save_structures(f, xdataset, xtermset, 0);

    return;
}


/*******************
*                  *
*  functionality   *
*                  *
*******************/


// provides compression of structure with additional operations
error_t
lraam_compression(lraam_t * xraam, int ** descr, set_t * xtermset, 
                  vector_t compression, value_t *recerr, bool_t learning) {
    error_t err;
    vector_t layer;    
    vector_t vect;
    object_t * obj;
    value_t tmperr;
    int i;

    assert(INITIALIZED == 1);
    assert(xraam != NULL);
    assert(descr != NULL);
    assert(*descr != NULL);
    assert(xtermset != NULL);
    assert(xtermset->type == TERMINAL);
    assert(xtermset->dimension == xraam->dimension);

    if (compression != NULL) vector_reset(compression, MAX_DIMENSION);
    if (recerr != NULL) *recerr = 0;
    
    // terminal
    if (**descr >= 0) {
	obj = set_get_object_by_id(xtermset, **descr);
	if (obj == NULL) return ERR_SYNTAX;
	if (compression != NULL) 
	    vector_copy(compression, obj->representation, obj->dimension);
	(*descr)++;
	return ERR_OK;
    }
    
    // left parenthesis
    if (**descr != LEFT_PAREN) return ERR_SYNTAX;
    (*descr)++;
    // temporary memory allocation
    layer = vector_alloc(ARITY * MAX_DIMENSION);    
    if (layer == NULL) return ERR_NOT_ENOUGH_MEM;
    vector_reset(layer, ARITY * MAX_DIMENSION);
    vect = vector_alloc(MAX_DIMENSION);
    if (vect == NULL) { vector_free(layer); return ERR_NOT_ENOUGH_MEM; }
    // compressing all ARITY substructures (maybe only terminals)
    for (i=0; i<ARITY; i++) {
	if ((err = lraam_compression(xraam, descr, xtermset, vect, &tmperr, learning)) != ERR_OK) {
	    vector_free(layer);
	    vector_free(vect);
	    return err;
	}
	if (recerr != NULL) *recerr += tmperr;
	layer_set_node(layer, vect, i);
    }
    // encoding, decoding, learning, reconstruction error
    vector_copy(xraam->input_layer, layer, ARITY * MAX_DIMENSION);
    lraam_encode(xraam);
    lraam_decode(xraam);
    if (compression != NULL) lraam_get_compressed(xraam, compression);
    if (learning == 1) lraam_update_weights(xraam);
    if (recerr != NULL) {
	tmperr = 0.0;
	for (i=0; i<ARITY; i++) {
	    tmperr += vector_sqredist(xraam->input_layer+(i*MAX_DIMENSION), 
	                              xraam->output_layer+(i*MAX_DIMENSION), xraam->dimension);
	}
	*recerr += tmperr;
    }
    // releasing memory
    vector_free(layer);
    vector_free(vect);
    // right parenthesis
    if (**descr != RIGHT_PAREN) return ERR_SYNTAX;
    (*descr)++;
    
    return ERR_OK;
}


// decodes representation into possible structure
error_t
lraam_decompression(lraam_t * xraam, vector_t repr, int maxdepth, set_t * xtermset, int * descr) {
    error_t err;
    int i;
    vector_t layer;
    vector_t vect;
    int * buffer;

    assert(xraam != NULL);
    assert(repr != NULL);
    assert(maxdepth >= 0);
    assert(xtermset != NULL);
    assert(xtermset->type == TERMINAL);
    assert(xraam->dimension == xtermset->dimension);
    assert(descr != NULL);

    // maybe it is representation of terminal
    i = set_get_term_by_repr(xtermset, repr, lraam_get_theta(xraam));
    if (i >= 0) {
	descr[0] = 1;
	descr[1] = i;
	return ERR_OK;
    }
    
    // in this case it should have been terminal
    if (maxdepth == 0) {
	descr[0] = 0;
	return ERR_SYNTAX;
    }
    
    // allocating temporary memory    
    if ((layer = vector_alloc(ARITY * MAX_DIMENSION)) == NULL) return ERR_NOT_ENOUGH_MEM;
    if ((vect = vector_alloc(MAX_DIMENSION)) == NULL) { 
	vector_free(layer);
	return ERR_NOT_ENOUGH_MEM;
    }
    if ((buffer = (int *)calloc(MAX_STRUCT_LENGTH, sizeof(int))) == NULL) {
	vector_free(layer);
	vector_free(vect);
	return ERR_NOT_ENOUGH_MEM;
    }
    
    // appending substructure descriptions to resulting description
    // starting with left parenthesis
    descr[0] = 1;
    descr[1] = LEFT_PAREN;
    lraam_set_compressed(xraam, repr);
    lraam_decode(xraam);
    vector_copy(layer, xraam->output_layer, ARITY * MAX_DIMENSION);
    for (i=0; i<ARITY; i++) {
	layer_get_node(layer, vect, i);
	err = lraam_decompression(xraam, vect, maxdepth-1, xtermset, buffer);
	if (err == ERR_OK) {
	    descr_append(descr, buffer);		    
	} else {
	    descr[0] = 0;
	    vector_free(vect);
	    vector_free(layer);
	    free(buffer);
	    return err;
	}
    }
    // append right parenthesis
    buffer[0] = 1;
    buffer[1] = RIGHT_PAREN;
    descr_append(descr, buffer);
    
    //releasing allocated temporary memory
    vector_free(vect);
    vector_free(layer);
    free(buffer);
    
    return ERR_OK;
}


// provides "no_of_iterations" presentations of dataset with additional operation
error_t
lraam_set_presentation(lraam_t * xraam, set_t * xdataset, set_t * xtermset, 
                       int no_of_iterations, value_t * recerr, bool_t learning, 
		       bool_t * encodables) {
    error_t err;
    int i, iter, n;
    int * ip, * A;
    object_t * obj, *tmp;
    vector_t vect;
    value_t tmperr, * errptr;
    
    assert(xraam != NULL);
    assert(xdataset != NULL);
    assert(xdataset->type == STRUCTURE);
    assert(xtermset != NULL);
    assert(xtermset->type == TERMINAL);
    assert(xdataset->dimension == xraam->dimension);
    assert(xtermset->dimension == xraam->dimension);
    assert(no_of_iterations > 0);

    // some initialization
    n = xdataset->no_of_objects;	// will be used often

    errptr = NULL;	// we will pass errptr to recursive calls of this function
    if (recerr != NULL) { errptr = &tmperr; *recerr=0; } // instead of asking each time

    tmp = NULL;	
    if (encodables != NULL) {	// in case encodable structures are desired..
	for (i=0; i<n; i++) encodables[i] = 0;	// no structure encodable by default
	tmp = object_alloc(STRUCTURE);		// useful when decompressing
	if (tmp == NULL) return ERR_NOT_ENOUGH_MEM;
    }
    
    if (learning == 0) no_of_iterations = 1;	// no other iterations needed

    // some memory allocation
    if ((vect = vector_alloc(MAX_DIMENSION)) == NULL)
	return ERR_NOT_ENOUGH_MEM;
    if ((A = (int *)calloc(n, sizeof(int))) == NULL) { // for dataset permutation
	vector_free(vect); 
	return ERR_NOT_ENOUGH_MEM; 
    }
    for (i=0; i<n; i++) A[i] = i;

    // iterations
    for (iter=0; iter<no_of_iterations; iter++) {
	if (learning == 1) permute(A, n);
	if (recerr != NULL) *recerr = 0.0;
	for (i=0; i<n; i++) {
	    // one learning iteration
	    tmperr = 0.0;	// only for sure
	    obj = set_get_object_by_id(xdataset, A[i]);
	    ip = obj->structure.description + 1; 	// dont use 0th length-determinating field
	    err = lraam_compression(xraam, &ip, xtermset, obj->representation, errptr, learning); // compression
	    if (err != ERR_OK) {	// an error occured
		vector_free(vect);
		free(A);
		if (encodables != NULL) object_free(tmp);
		return err;
	    }

	    if (recerr != NULL) *recerr += sqrt(tmperr);	// collecting total reconstruction error

	    // encodables are tested during last presentation cycle
	    if ((iter == no_of_iterations - 1) && (encodables != NULL)) {
		// get decoded structure
		err = lraam_decompression(xraam, obj->representation, obj->structure.depth+1, // compressed representation
		                          xtermset, tmp->structure.description); // is in 'vect'
		if (err == ERR_OK) {
		    if (!descr_cmp(tmp->structure.description, obj->structure.description)) encodables[A[i]] = 1;
		} else if (err == ERR_SYNTAX) {
		    // .. happens
		} else {
		    // ERR_NOT_ENOUGH_MEM
		    vector_free(vect);
		    free(A);
		    object_free(tmp);
		    return err;
		}
	    }
	}
    }
    
    // free memory
    vector_free(vect);
    free(A);
    if (encodables != NULL) object_free(tmp);

    return ERR_OK;
}


/*******************
*                  *
*  generalization  *
*                  *
*******************/


// write to A first partition of sum: A = (1,1, .. ,sum-AR+1)
static void 
partition_init(int * A, int sum) {
    int i;
    
    assert(A != NULL);
    assert(sum >= ARITY);
    
    for (i=0; i<ARITY; i++) A[i] = 1;
    A[ARITY-1] = sum-ARITY+1;
    
    return;
}


// compute next partition
static int 
partition_next(int * A, int sum) {
    int i;

    assert(A != NULL);
    assert(sum >= ARITY);

    if (sum == ARITY) return 0;
    if (A[ARITY-1] == 1) {
	for (i=ARITY-1; A[i] == 1; i--);
	if (i==0) return 0; // A == (x,1, .. ,1)
	A[ARITY-1] = A[i]-1;
	A[i] = 1;
	A[i-1]++;
    } else {
	A[ARITY-1]--;
	A[ARITY-2]++;
    }

    return 1;        
}


// return initial choice of substructures according to a given partition
static int
positions_init(int * starts, int * positions, int * partition) {
    int i;
    int count;

    assert(starts != NULL);
    assert(positions != NULL);
    assert(partition != NULL);
    
    count = 1;
    for (i=0; i<ARITY; i++) {
	// list of structures of length N begins at start[N] and ends at start[N+1]-1
	// so, if start[N+1] - start[N] == 0, there is no structure of length N
	count *= ((starts[partition[i]+1] - starts[partition[i]]) != 0 ? 1 : 0);
	positions[i] = starts[partition[i]];
    }
    
    return count;	// used to return no_of_structures that we can get when combining, 
			// but it can be a large number
}


// compute next choice of substructures
// starts[i] - id of first structure with depth i
// positions[j] - id of jth chosen substructure
// parition - partition of an integer sum - total depth of structures
static int
positions_next(int * starts, int * positions, int * partition) {
    int i, ovf;

    assert(starts != NULL);
    assert(positions != NULL);
    assert(partition != NULL);
    
    ovf = 1;	// i think its an abbreviation of "overflow"
		// its long time since i have write the code
    for (i = ARITY-1; i>=0 && ovf; i--) {
	ovf = 0;
	positions[i]++;
	if (positions[i] == starts[partition[i]+1]) {
	    positions[i] = starts[partition[i]];
	    ovf = 1;
	}
    }

    return 1-ovf;
}


static void
partition_fprintf(int * partition) {
    int i;
    for (i=0; i<ARITY; i++) fprintf(stderr, "%d ", partition[i]); fprintf(stderr, "\n");
    return;
}


static void
positions_fprintf(int * positions) {
    int i;
    fprintf(stderr, "\t"); for (i=0; i<ARITY; i++) fprintf(stderr, "%d ", positions[i]); fprintf(stderr, "\n");
    return;
}


// generalization procedure
error_t
lraam_generalize(lraam_t * xraam, set_t * xgenset, set_t * xdataset, set_t * xtermset, 
    int maxlength, int maxdepth, bool_t verbose) {

    int * starts;	// starts[i] = index at which sequence of structures with the length=i starts
    int * partition;	// partition of a number L=length on A=arity sumands
    int * positions;	// possible choices of structures with respect to given partition

    int * buffer, * nbuffer;
    int i, j, k;
    int len, last, change, depth, dimension;
    error_t err, tmperr;
    vector_t repr;
    object_t * obj;

    // neccessary conditions
    assert(xraam != NULL);
    assert(xgenset != NULL);
    assert(xgenset->type == STRUCTURE);
    assert(xdataset != NULL);
    assert(xdataset->type == STRUCTURE);
    assert(xtermset != NULL);
    assert(xtermset->type == TERMINAL);
    assert(xraam->dimension == xgenset->dimension);
    assert(xraam->dimension == xdataset->dimension);
    assert(xraam->dimension == xtermset->dimension);

    // set bounds
    if (maxlength < 0) maxlength = MAX_STRUCT_LENGTH/3 - 1;	// dont forget on parenthesis
    if (maxdepth < 0) maxdepth = MAX_STRUCT_LENGTH/3 - 1;	// ( a ( a ... ( a n ) ... ) )
    // 3 * DEPTH + 1 = MAX_STRUCT_LENGTH 	=> little assurance

    // since now we can exit only because of allocation problem
    err = ERR_NOT_ENOUGH_MEM;

    // initialization of variables
    dimension = xraam->dimension;
    if ((repr = vector_alloc(MAX_DIMENSION)) == NULL) return err;
    if ((partition = (int *)calloc(ARITY, sizeof(int))) == NULL) { vector_free(repr); return err; }
    if ((positions = (int *)calloc(ARITY, sizeof(int))) == NULL) { vector_free(repr); free(partition); return err; }
    if ((starts = (int *)calloc(MAX_STRUCT_LENGTH, sizeof(int))) == NULL) {
	vector_free(repr); free(partition); free(positions);
	return err;
    }
    if ((buffer = (int *)calloc(MAX_STRUCT_LENGTH, sizeof(int))) == NULL) {
	vector_free(repr); free(partition); free(positions); free(starts);
	return err;
    }
    if ((nbuffer = (int *)calloc(MAX_STRUCT_LENGTH, sizeof(int))) == NULL) {
	vector_free(repr); free(partition); free(positions); free(starts); free(buffer);
	return err;
    }
    starts[0] = 0;	// no structures with length = 0
    starts[1] = 0;
    
    // adding terminals to new set
    buffer[0] = 1;
    for (i=0; i<xtermset->no_of_objects; i++) {
	// description of ith terminal
	buffer[1] = i;
	// adding this terminal-structure to xgenset
	if (set_add_object(xgenset, buffer, xtermset->objects[i]->representation) == -1) {
	    // not enough memory when adding object
	    vector_free(repr); free(partition); free(positions); free(starts); free(buffer); free(nbuffer);
	    return err;
	};
    }
    
    // we cannot get a structure with 1 < length < ARITY
    starts[2] = xgenset->no_of_objects;
    for (len = 3; len <= ARITY; len++) starts[len] = starts[len-1];

			    // VERBOSE MODE * VERBOSE MODE * VERBOSE MODE //
			    if (verbose) fprintf(stderr, "generalization");

    // generalization
    len = ARITY;
    last = 1;
    while (len <= last*ARITY && len <= maxlength) {
	change = 0;
	// for each positive partition of 'len' on 'arity' sumands
        partition_init(partition, len);
	do {
//	    partition_fprintf(partition);
	    // for each choice of substructures for given partition
	    i = positions_init(starts, positions, partition);
	    if (i==0) continue; // there is no choice for such a partition of lengths of substructures
	    do {
//		positions_fprintf(positions);
		// creates new structure and copy partial representations to input layer
		depth = 0;
		buffer[0] = 1;	// length
		buffer[1] = LEFT_PAREN;
		for (i=0; i<ARITY; i++) {
		    obj = set_get_object_by_id(xgenset, positions[i]);
		    if (depth < obj->structure.depth) depth = obj->structure.depth; // depth = longest path from root to a node
		    j = descr_append(buffer, obj->structure.description); // collecting increasing length
		    lraam_set_node(xraam, obj->representation, i); // representation of added structure
		}
		buffer[j+1] = RIGHT_PAREN;
		buffer[0] = j+1;
		depth++; // one additional edge from root to the substructure with highest depth
		if (depth > maxdepth) {
		    continue;
		}

		// get compressed representation
		lraam_encode(xraam);
		lraam_get_compressed(xraam, repr);

		// decompression
		nbuffer[0] = 0;
		tmperr = lraam_decompression(xraam, repr, depth, xtermset, nbuffer);
		if (tmperr == ERR_SYNTAX) continue;
		if (tmperr == ERR_NOT_ENOUGH_MEM) {
		    vector_free(repr); free(partition); free(positions); free(starts); free(buffer); free(nbuffer);
		    return tmperr;
		}
		if (descr_cmp(buffer, nbuffer)) continue;	// not the same structure after decompressing
		// decoded successfully
		if ((k = set_add_object(xgenset, nbuffer, repr)) == -1) {
		    vector_free(repr); free(partition); free(positions); free(starts); free(buffer); free(nbuffer);
		    return ERR_NOT_ENOUGH_MEM;
		}
//		struct_fprintf(stderr, set_get_object_by_id(xgenset, k), xtermset, 0);
		change = 1;
	    } while (positions_next(starts, positions, partition));
	} while (partition_next(partition, len));

				// VERBOSE MODE * VERBOSE MODE * VERBOSE MODE //
				if (change) if (verbose) fprintf(stderr, ".");

	if (change) last = len;	// in this run we added at least one structure to the genset
	len++;
	// we must look to the future
	for (i=len; i<=last*ARITY; i++) starts[i] = xgenset->no_of_objects;
    }


				// VERBOSE MODE * VERBOSE MODE * VERBOSE MODE //
				if (change) if (verbose) fprintf(stderr, "done\n");

    // releasing temporary memory
    vector_free(repr); 
    free(partition); 
    free(positions); 
    free(starts); 
    free(buffer); 
    free(nbuffer);

    // all is now ok
    return ERR_OK;
}
