/* 
 * linear recursive autoassociative memory 
 * - header file
 * by Michal Pokorny
 *
 * version 3.0 
 */
 
#ifndef LRAAM_H
#define LRAAM_H

// definitions of types
// -------------------------------------------------------------------------------
enum enum_object_types 					// object types
{ TERMINAL, STRUCTURE };

enum enum_errors					// errors
{ ERR_OK, ERR_IO_FEW_PARAMS, ERR_SYNTAX, ERR_NOT_ENOUGH_MEM };

enum enum_special_codes					// encoding structures
{ LEFT_PAREN = -4, RIGHT_PAREN }; 

enum enum_special_terminals				// term matching
{ NO_MATCHING_TERMINAL = -2, TWO_MATCHING_TERMINALS };

typedef int bool_t;
typedef int error_t;

typedef double value_t;
typedef value_t * vector_t;
typedef value_t *** matrix_t;

typedef struct {
    char * label;		// terminal label
} term_t;

typedef struct {
    int * description;		// structure description
    int depth;			// structure depth - depth of a terminal is = 0
} struct_t;

typedef struct {
    union {
	term_t terminal;	// object can be terminal 
	struct_t structure;	// or structure
    };
    int type;			// object type: TERMINAL | STRUCTURE
    vector_t representation;	// terminal / structure representation
    int dimension;		// dimension of representation vector
    bool_t assigned; 		// 0 iff representation was not assigned yet
} object_t;

typedef struct {
    object_t ** objects;	// array of pointers to terminals / structures
    int type;		// set type: TERMINAL | STRUCTURE set
    int array_length; 	// number of blocks allocated in object array
    int no_of_objects; 	// number of objects in array
    int dimension;	// dimension of representation vector for all objects
} set_t;

typedef struct {
    int dimension;	// vector dimension
    value_t theta; 	// treshold
    value_t eta; 	// learning rate
    matrix_t W; 	// array of weight matrices from each node to the whole
    vector_t input_layer; 	// input layer
    vector_t output_layer; 	// decoded compressed representation
    vector_t compression_layer;	// compressed representation of input layer
} lraam_t;


// initialization
// -------------------------------------------------------------------------------
void randgen_init(unsigned int, int);
void lraamlib_init(int, int, int, int, int, int);

// misc
// -------------------------------------------------------------------------------
double intrand(double, double, int);
void permute(int *, int);


// manipulation with value_t typed variables
// -------------------------------------------------------------------------------
void value_fprintf(FILE *, value_t);
error_t value_fscanf(FILE *, value_t *);

// manipulation with vectors
// -------------------------------------------------------------------------------
vector_t vector_alloc(int);
void vector_free(vector_t);
void vector_reset(vector_t, int);
void vector_intrand(vector_t, int, double, double);
void vector_randomize(vector_t, int);
void vector_copy(vector_t, vector_t, int);
vector_t vector_clone(vector_t, int);
void vector_fprintf(FILE * f, vector_t, int, char *);
void vector_write(FILE * f, vector_t, int);
error_t vector_fscanf(FILE *, vector_t, int);
error_t vector_read(FILE *, vector_t, int);
value_t vector_sqredist(vector_t, vector_t, int);
 
// manipulation with layers
// -------------------------------------------------------------------------------
void layer_set_node(vector_t, vector_t, int);
void layer_get_node(vector_t, vector_t, int);
void layer_fprintf(FILE * f, vector_t, int, char *, char *);
error_t layer_fscanf(FILE *, vector_t, int);
 
// manipulation with matrices
// -------------------------------------------------------------------------------
matrix_t matrix_alloc();
void matrix_free(matrix_t);
void matrix_reset(matrix_t);
void matrix_copy(matrix_t, matrix_t);
void matrix_intrand(matrix_t, double, double);
void matrix_randomize(matrix_t);
void matrix_fprintf(FILE *, matrix_t, int);
void matrix_write(FILE *, matrix_t, int);
error_t matrix_fscanf(FILE *, matrix_t, int);
error_t matrix_read(FILE *, matrix_t, int);

// manipulation with terminals or structures
// -------------------------------------------------------------------------------
object_t * object_alloc(int);
object_t * object_clone(object_t *);
void object_free(object_t *);
void object_reset(object_t *);
void object_label(object_t *, void *);

void object_assign_representation(object_t *, vector_t);
void object_reset_representation(object_t *);
void object_change_dimension(object_t *, int);

void term_fprintf(FILE *, object_t *);
error_t term_fscanf(FILE *, int, char *, vector_t);
error_t struct_fprintf(FILE *, object_t *, set_t *, bool_t);

// manipulation with structure description
// -------------------------------------------------------------------------------
int descr_length(int *);
int descr_depth(int *);
int descr_append(int *, int *);
int descr_cmp(int *, int *);
int descr_copy(int *, int *);

error_t str_to_descr(char *, int *, set_t *);
void descr_to_str(int *, char *, set_t *);
error_t descr_validate(int *);

// manipulation with sets
// -------------------------------------------------------------------------------
set_t * set_alloc(int, int);
error_t set_realloc(set_t *);
void set_free(set_t *);
void set_reset(set_t *);
void set_change_dimension(set_t *, int);
int set_add_object(set_t *, void *, vector_t);

// sets miscellaneous
// -------------------------------------------------------------------------------
int set_get_term_by_label(set_t *, char *);
int set_get_term_by_repr(set_t *, vector_t, value_t);
object_t * set_get_object_by_id(set_t *, int);

void set_save_terminals(FILE *, set_t *);
void set_save_structures(FILE *, set_t *, set_t *, bool_t);
set_t * set_load_terminals(FILE *, int);
set_t * set_load_structures(FILE *, set_t *);

// manipulation with lraam
// -------------------------------------------------------------------------------
void lraam_reset_input_layer(lraam_t *);
void lraam_reset_output_layer(lraam_t *);
void lraam_reset_compression_layer(lraam_t *);
void lraam_randomize_weights(lraam_t *);

lraam_t * lraam_alloc();
lraam_t * lraam_load(FILE *);
void lraam_save(FILE *, lraam_t *);
void lraam_free(lraam_t *);
void lraam_change_dimension(lraam_t *, int);
void lraam_set_theta(lraam_t *, value_t);
value_t lraam_get_theta(lraam_t *);
void lraam_set_eta(lraam_t *, value_t);	
value_t lraam_get_eta(lraam_t *);

void lraam_set_node(lraam_t *, vector_t, int);
void lraam_get_modif_node(lraam_t *, vector_t, int);
void lraam_set_compressed(lraam_t *, vector_t);
void lraam_get_compressed(lraam_t *, vector_t);

void lraam_encode(lraam_t *);
void lraam_decode(lraam_t *);
void lraam_update_weights(lraam_t *);

// backup functions
// -------------------------------------------------------------------------------
error_t experiment_load(FILE *, lraam_t **, set_t **, set_t **);
void experiment_save(FILE *, lraam_t *, set_t *, set_t *, bool_t);

// lraam functionality
// -------------------------------------------------------------------------------
error_t lraam_compression(	lraam_t *, 	int **, 	set_t *, 
				vector_t, 	value_t *, 	bool_t);

error_t lraam_decompression(	lraam_t *, 	vector_t, 	int, 
				set_t *, 	int *);	

error_t lraam_set_presentation(	lraam_t *, 	set_t *, 	set_t *, 
				int, 		value_t *,	bool_t,
				bool_t *);		

error_t lraam_generalize(	lraam_t *, set_t *, set_t *, set_t *,
			 	int, int, bool_t);
 
#endif /* LRAAM_H */
