/*!\file lraam_output.c
 * 
 * \author Michal Pokorny
 * \date 02/2006 - 03/2006
 *
 * \brief Utility that can display parts of experiment given on standard input
 *        in readable form.
 * 
 * User is able to extract termset or dataset and see weight matrix in readable
 * form (not binary data as it is stored in experiment file).
 */
 
#include <getopt.h>
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "lraam.h"

#define MAX_PATH_LENGTH 	128

lraam_t * raam;
set_t * dataset;
set_t * termset;
int experiment_loaded = 0;

// command line options defaults
// -----------------------------------------------------------------------------
int print_all = 1;


// prints usage of this program
// -----------------------------------------------------------------------------
void
print_usage () {

    printf ("Usage:\n");
    printf ("  lraam_output [OPTIONS]\n");
    printf ("\nOptions:\n");
    printf ("  -t, --termset\n");
    printf ("       Print terminals and their representations.\n");
    printf ("  -d, --dataset\n");
    printf ("       Print structures from dataset.\n");
    printf ("  -m, --matrix\n");
    printf ("       Print current weight matrix in readable form. Do not use\n");
    printf ("       this output for further simulations - shown data are not\n");
    printf ("       strictly the same as those stored in memory.\n");
    printf ("  -h, --help\n");
    printf ("       Display this help and exit.\n");
    
    return;
}


// release memory allocated when loading experiment
// -----------------------------------------------------------------------------
void free_all() {

    lraam_free(raam);
    set_free(dataset);
    set_free(termset);

    return;
}


// loads experiment from stdin
// -----------------------------------------------------------------------------
void
load_exp() {
    error_t err;
    
    // loading experiment
    err = experiment_load(stdin, &raam, &dataset, &termset);
    if (err != ERR_OK) {
	fprintf(stderr, "ERROR %d: cannot load experiment.\n\n", err);
	exit(err);
    }
    
    experiment_loaded = 1;
}


// parses command line options
// -----------------------------------------------------------------------------
void
parse_options(int argc, char **argv) {
    int c;
    
    static const struct option long_options[] = {
	{"termset", no_argument, NULL, 't'},
	{"dataset", no_argument, NULL, 'd'},
	{"matrix", no_argument, NULL, 'm'},
	{"help", no_argument, NULL, 'h'},
	{0, 0, 0, 0}
    };

    static const char short_options[] = "tdmh";

    while(1) {
	c = getopt_long(argc, argv, short_options, long_options, NULL);
	if (c == -1) break;
	switch (c) {
	    case 't': 
		if (!experiment_loaded) load_exp();
		set_save_terminals(stdout, termset);
		print_all = 0;
		break;
	    case 'd': 
		if (!experiment_loaded) load_exp();
		set_save_structures(stdout, dataset, termset, 0);
		print_all = 0;
		break;
	    case 'm': 
		if (!experiment_loaded) load_exp();
		matrix_fprintf(stdout, raam->W, raam->dimension);
		fprintf(stdout, "\n");
		print_all = 0;
		break;
	    case 'h':
	    case '?':
	    default:
		print_usage();
		if (experiment_loaded) free_all();
		exit(1);
	}
    }
    
    return;
}


// main
// -----------------------------------------------------------------------------
int
main(int argc, char **argv) {

    // get command line options and load experiment if needed
    parse_options(argc, argv);

    // if no option was set
    if (!experiment_loaded) load_exp();

    // if no params were set, print state of experiment in readable form
    if (print_all) {
	experiment_save(stdout, raam, dataset, termset, 1);
    }

    // releasing memory
    free_all();

    return 0;
}
